import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * The game board.
 * 
 * @author mik
 * @author Michael Berry
 * @version 1.0
 */
public class Board extends World
{
    public static final String PORT = null /* "COM3" */;
    private static GreenPicoBoard board;
    private Paddle paddle;
    /** The number of levels */
    public final int levelNum = 5;
    private int level;
    /** The number of lives a player starts with */
    public final int lifeNum = 3;
    private int lives, scoreNum;
    private Text lifeText, scoreText;
    private boolean nextLevel;
    
    /**
     * Create a new board
     */
    public Board()
    {    
        super(460, 520, 1);
        setPaintOrder (BigText.class, Powerup.class, Ball.class, Smoke.class );
        Greenfoot.setSpeed(50);
        
        paddle = new Paddle();
        addObject ( paddle, getWidth() / 2, getHeight() - 40);
        
        lives=lifeNum;
        lifeText = new Text();
        lifeText.setText("Lives: 3");
        addObject(lifeText, 70, getHeight()-10);
        
        scoreNum = 0;
        scoreText = new Text();
        scoreText .setText("Score: 0");
        addObject(scoreText, 140, getHeight()-10);        
        
        level=1;
        setupLevel();
        
        //Initialise the images now to avoid a delay later
        Explosion.initialiseImages();
    }
    
    public GreenPicoBoard getPicoBoard() {
        return board;
    }
        
    /**
     * Leave this line in place!
     */
    public void started() {
        board = new GreenPicoBoard(PORT);
    }
    
    /**
     * Leave this line in place!
     */
    public void stopped() {
        board.close();
    }
    
    /**
     * Checks to see where the ball is, only sets the next level up if the ball is
     * in the lower area of the board. Stop if no lives left.
     */
    public void act()
    {
        if(nextLevel) {
            List<Ball> balls = getObjects(Ball.class);
            boolean r = true;
            for(Ball ball : balls) {
                if(ball.getY()<400) r = false;
            }
            if(r) setupLevel();
        }
        if(lives==0) gameOver();
    }
    
    /**
     * Adds a number to the score
     * @param n the number to add to the score
     */
    public void addScore(int n)
    {
        scoreNum += n;
        scoreText.setText("Score: " + scoreNum);
    }
    
    /**
     * Adds 1 life
     */
    public void addLife()
    {
        lives++;
        lifeText.setText("Lives: " + lives);
    }
    
    /**
     * Sets up a specified level
     */
    public void setupLevel()
    {
        nextLevel = false;
        removeBlocks();
        BigText txt = new BigText();
        addObject(txt, getWidth()/4, (getHeight()/4)*3);
        txt.setText("Level " + level);
        if(level==1) {
            for(int i=20 ; i<getWidth()-20 ; i+= 40) {
                for(int j=20 ; j <= 40 ; j += 20) {
                    addObject(new RedBlock(), i, j);
                }
            }
        }
        else if(level==2) {
            for(int i=20 ; i<getWidth()-20 ; i+= 40) {
                for(int j=20 ; j <= 40 ; j += 20) {
                    addObject(new RedBlock(), i, j);
                }
            }
            addObject(new Bomb(), getWidth()/2, 70);
            for(int i=50 ; i<getWidth()-50 ; i+= 40) {
                addObject(new GreenBlock(), i, 100);
            }
        }
        else if(level==3) {
            int j=50;
            for(int i=50 ; i<getWidth()/2 ; i+= 30) {
                j+=20;
                addObject(new BlueBlock(), i, j);
            }
            for(int i=getWidth()/2 ; i<getWidth()-50 ; i+=30) {
                addObject(new BlueBlock(), i, j);
                j-=20;
            }
            addObject(new Bomb(), getWidth()/2, 70);
            addObject(new GreenBlock(), getWidth()/2, 50);
            addObject(new GreenBlock(), getWidth()/2, 90);
            addObject(new RedBlock(), 440, 70);
            addObject(new RedBlock(), 410, 70);
            addObject(new RedBlock(), 30, 70);
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new GreenBlock(), i, 7);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new RedBlock(), i, 200);
            }
        }
        else if(level==4) {
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new BlueBlock(), i, 7);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                if(i%20==0) addObject(new Bomb(), i, 40);
                else addObject(new GreenBlock(), i, 40);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new GreenBlock(), i, 80);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                if(i%20==0) addObject(new Bomb(), i, 120);
                else addObject(new GreenBlock(), i, 120);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new RedBlock(), i, 160);
            }
        }
        else if(level==5) {
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new BlueBlock(), i, 7);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                if(i%20==0) addObject(new Bomb(), i, 40);
                else addObject(new BlueBlock(), i, 40);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new BlueBlock(), i, 80);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                if(i%20==0) addObject(new Bomb(), i, 120);
                else addObject(new BlueBlock(), i, 120);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new BlueBlock(), i, 160);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new GreenBlock(), i, 190);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new RedBlock(), i, 210);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new RedBlock(), i, 230);
            }
            for(int i=20 ; i<getWidth() ; i += 30) {
                addObject(new BlueBlock(), i, 250);
            }
        }
    }
    
    /**
     * Called when the level has been completed. Advances to the next
     * level and checks whether all levels have been completed.
     */
    public void nextLevel()
    {
        if(level==levelNum) {
            completed();
        }
        else {
            level++;
            nextLevel = true;
            addScore(level*33);
        }
    }
    
    /**
     * Called when all levels have been completed.
     */
    public void completed()
    {
        for(Actor actor : (List<Actor>)getObjects(null)) {
            removeObject(actor);
        }
        
        BigText txt = new BigText();
        addObject(txt, 5, (getHeight()/4)*3);
        txt.setText("Congratulations!");
        txt = new BigText();
        addObject(txt, 5, ((getHeight()/4)*3)+70);
        txt.setText("Score: " + scoreNum);   
        
        Greenfoot.stop();
    }
    
    /**
     * Called when all lives have been lost
     */
    public void gameOver()
    {
        for(Smoke smoke : (List<Smoke>)getObjects(Smoke.class)) {
            removeObject(smoke);
        }
        for(BigText text : (List<BigText>)getObjects(BigText.class)) {
            removeObject(text);
        }        
        
        BigText txt = new BigText();
        addObject(txt, 5, (getHeight()/4)*3);
        txt.setText("Game Over!");
        txt = new BigText();
        addObject(txt, 5, ((getHeight()/4)*3)+70);
        txt.setText("Score: " + scoreNum);
        
        Greenfoot.stop();
    }
    
    /**
     * Removes all blocks from the world
     */
    public void removeBlocks()
    {
        List<Actor> blocks = getObjects(Block.class);
        for(Actor block : blocks) {
            removeObject(block);
        }
    }
    
    /**
     * Called when the ball has not been caught by the paddle.
     */
    public void ballIsOut()
    {
        takeLife();
        paddle.newBall();
    }
    
    /**
     * Takes a life from the player
     */
    private void takeLife()
    {
        lives--;
        lifeText.setText("Lives: " + lives);
    }
}
