import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * The game paddle. It is keyboard controlled (left, right, space). It also 
 * generates a new ball when it is created itself.
 * 
 * @author mik
 * @version 1.0
 */
public class Paddle extends Actor
{
    private Ball myBall;  // used before ball gets released

    /**
     * When the paddle gets created, create a ball as well.
     */
    public void addedToWorld(World world)
    {
        newBall();
    }
    
    /**
     * Act - do whatever the Paddle wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        setLocation((int)(((Board)getWorld()).getPicoBoard().getSlider()*4.6), getY());
        if(myBall != null) {
            myBall.setLocation(getX(),getY()-getImage().getHeight());
        }
        if (haveBall() && (((Board)getWorld()).getPicoBoard().isButtonPressed())) {
            releaseBall();
        }
        checkPowerup();
        
    }
    
    /**
     * Check for any powerups hitting the bat
     */
    private void checkPowerup()
    {
        Powerup p = (Powerup)getOneIntersectingObject(Powerup.class);
        if(p==null) return;
        Board board = (Board)getWorld();
        board.removeObject(p);
        
        if(p instanceof BallPower) {
            if(!haveBall()) {
                Ball ball = new Ball();
                board.addObject(ball, board.getWidth()/2, board.getHeight()/2);
                ball.release();
            }
        }
        else if(p instanceof SpeedPower) {
            for(Ball ball : (List<Ball>)board.getObjects(Ball.class)) {
                ball.accelerate();
            }
        }
        else if(p instanceof LifePower) {
            board.addLife();
        }  
        else if(p instanceof Bonus) {
            board.addScore(200);
        }        
        else {
            throw new RuntimeException("Unknown powerup...");
        }
    }
    
    public void newBall()
    {
        myBall = new Ball();
        getWorld().addObject (myBall, getX(), getY()-20);
    }
        
    public boolean haveBall()
    {
        return myBall != null;
    }
        
    public void releaseBall()
    {
        myBall.release();
        myBall = null;
    }
        
}
